@@ -118,9 +118,10 @@
 	  (do ((i 0 (+ i 1))
 	       (sum 0.0 0.0))
 	      ((= i N))
-	    (do ((k 0 (+ k 1)))
+	    (do ((k 0 (+ k 1))
+		 (cin (/ (* 2.0 0+1.0i pi i) N)))
 		((= k N))
-	      (set! sum (+ sum (* (vals k) (exp (/ (* 2.0 0+1.0i pi k i) N))))))
+	      (set! sum (+ sum (* (vals k) (exp (* cin k))))))
 	    (set! (w i) (magnitude sum))
 	    (set! pk (max pk (w i))))
 	  ;; scale to 1.0 (it's usually pretty close already, that is pk is close to 1.0)
